const ethUtil = require('ethereumjs-util');
const sigUtil = require('@metamask/eth-sig-util');

const app = require('j7/app');

async function authVerifySignature(session) {
  try {
    const msgParams = {
      types: {
        EIP712Domain: [
          { name: 'name', type: 'string' },
          { name: 'version', type: 'string' },
        ],
        set: [
          { name: 'tips', type: 'string' },
          { name: 'nonce', type: 'string' },
        ]
      },
      primaryType: 'set',
      domain: {
        name: 'Auth',
        version: '1',
      },
      message: {
        tips: session.request('tips'),
        nonce: session.request('nonce')
      }
    };
    const recovered = sigUtil.recoverTypedSignature(
      {data:
       msgParams,
       signature: session.request('signature'),
       version: 'V4'});
    session.rspData(
      {
        'recovered': recovered
      });
  } catch (err) {
    session.rspErr(1, err);
  }
}

function init() {
  app.registerHandler('BcService', 'authVerifySignature', authVerifySignature);
}

exports.init = init;
